package com.gree.mall.manager.logic.policy;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.db.sql.Order;
import com.gree.mall.manager.bean.policy.WorkerManagerBean;
import com.gree.mall.manager.bean.policy.WorkerManagerDetail;
import com.gree.mall.manager.enums.ExamineWorkerStatusEnum;
import com.gree.mall.manager.enums.OrderStatusEnum;
import com.gree.mall.manager.enums.workorder.OrderBaseStatusEnum;
import com.gree.mall.manager.exception.RemoteServiceException;
import com.gree.mall.manager.plus.entity.PgOrderBase;
import com.gree.mall.manager.plus.entity.WebsitUser;
import com.gree.mall.manager.plus.entity.WorkerImg;
import com.gree.mall.manager.plus.entity.WorkerPolicy;
import com.gree.mall.manager.plus.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
public class WorkerManagerLogic {

    @Autowired
    WebsitUserService websitUserService;

    @Autowired
    PgOrderBaseService pgOrderBaseService;

    @Autowired
    WorkerImgService workerImgService;

    @Autowired
    WorkerPolicyService workerPolicyService;

    @Autowired
    UserService userService;


    public void stop(String id) {

        WebsitUser websitUserBean = websitUserService.getById(id);
        if (!websitUserBean.getExamineStatus().equals(ExamineWorkerStatusEnum.OK.getKey()))
            throw new RemoteServiceException("师傅不处于审核通过");

        WebsitUser websitUser = new WebsitUser();
        websitUser.setId(id);
        websitUser.setExamineStatus(ExamineWorkerStatusEnum.STOP.getKey());
        websitUser.updateById();
    }

    public void activation(String id) {

        WebsitUser websitUserBean = websitUserService.getById(id);
        if (!websitUserBean.getExamineStatus().equals(ExamineWorkerStatusEnum.STOP.getKey()))
            throw new RemoteServiceException("师傅不处于暂停状态");

        WebsitUser websitUser = new WebsitUser();
        websitUser.setId(id);
        websitUser.setExamineStatus(ExamineWorkerStatusEnum.POLICY_WAIT.getKey());
        websitUser.updateById();
    }

    public void quit(String id) {

        WebsitUser websitUserBean = websitUserService.getById(id);
        if (!websitUserBean.getExamineStatus().equals(ExamineWorkerStatusEnum.OK.getKey()))
            throw new RemoteServiceException("师傅不处于审核通过");


        if (pgOrderBaseService.lambdaQuery()
                .and(item -> item.eq(PgOrderBase::getWorkerId,websitUserBean.getUserId()).or().like(PgOrderBase::getWorkerIdList,websitUserBean.getUserId()))
                .eq(PgOrderBase::getWebsitId,websitUserBean.getWebsitId())
                .notIn(PgOrderBase::getOrderStatus, OrderBaseStatusEnum.YQX.getKey(),OrderBaseStatusEnum.YJS.getKey(),
                        OrderBaseStatusEnum.YWG.getKey(),OrderBaseStatusEnum.FWQX.getKey(),OrderBaseStatusEnum.YWGO.getKey(),
                        OrderBaseStatusEnum.YGBGD.getKey(),OrderBaseStatusEnum.FGFZF.getKey(),OrderBaseStatusEnum.YZFGD.getKey(),
                        OrderBaseStatusEnum.FZXBWG.getKey(),OrderBaseStatusEnum.FBBWG.getKey(),OrderBaseStatusEnum.LSXX.getKey(),
                        OrderBaseStatusEnum.PLGB.getKey(),OrderBaseStatusEnum.ZCGB.getKey()
                        ).count() > 1) {

            throw new RemoteServiceException("师傅存在未完工工单，请改派后再离职");
        }


        WebsitUser websitUser = new WebsitUser();
        websitUser.setId(id);
        websitUser.setExamineStatus(ExamineWorkerStatusEnum.QUIT.getKey());
        websitUser.updateById();
    }

    public void againQuit(String id) {

        WebsitUser websitUserBean = websitUserService.getById(id);
        if (!websitUserBean.getExamineStatus().equals(ExamineWorkerStatusEnum.QUIT.getKey()))
            throw new RemoteServiceException("师傅不处于离职状态");

        WebsitUser websitUser = new WebsitUser();
        websitUser.setId(id);
        websitUser.setExamineStatus(ExamineWorkerStatusEnum.POLICY_WAIT.getKey());
        websitUser.updateById();
    }

    @Transactional(rollbackFor = Exception.class)
    public void update(WorkerManagerBean workerManagerBean) {

        workerManagerBean.updateById();


        workerManagerBean.getUser().updateById();

        workerImgService.lambdaUpdate().eq(WorkerImg::getWorkerId,workerManagerBean.getUserId())
                .remove();


        for (WorkerImg workerImg : workerManagerBean.getWorkerImgs()) {
            workerImg.setWorkerId(workerManagerBean.getUserId());
        }


        workerImgService.saveOrUpdateBatch(workerManagerBean.getWorkerImgs());

        workerPolicyService.lambdaUpdate().eq(WorkerPolicy::getWebsitUserId,workerManagerBean.getId()).remove();


        for (WorkerPolicy workerPolicy : workerManagerBean.getWorkerPolicies()) {
            workerPolicy.setWorkerId(workerManagerBean.getUserId());
            workerPolicy.setWebsitUserId(workerManagerBean.getId());
        }

        workerPolicyService.saveOrUpdateBatch(workerManagerBean.getWorkerPolicies());


    }

    @Transactional(rollbackFor = Exception.class)
    public WorkerManagerDetail detail(String id) {

        WebsitUser websitUser = websitUserService.getById(id);

        WorkerManagerDetail workerManagerDetail = BeanUtil.toBean(websitUser, WorkerManagerDetail.class);

        workerManagerDetail.setUser(userService.getById(workerManagerDetail.getUserId()));

        List<WorkerImg> list = workerImgService.lambdaQuery().eq(WorkerImg::getWorkerId, websitUser.getUserId()).list();

        List<WorkerPolicy> workerPolicies = workerPolicyService.lambdaQuery().eq(WorkerPolicy::getWebsitUserId, websitUser.getId()).list();

        workerManagerDetail.setWorkerImgs(list);
        workerManagerDetail.setWorkerPolicies(workerPolicies);

        return workerManagerDetail;

    }

    @Transactional(rollbackFor = Exception.class)
    public void examineWait(WorkerManagerBean workerManagerBean) {

        //todo 检测保险情况 自建，第三方网点

        workerManagerBean.updateById();


        workerManagerBean.getUser().updateById();

        workerImgService.lambdaUpdate().eq(WorkerImg::getWorkerId,workerManagerBean.getUserId())
                .remove();


        for (WorkerImg workerImg : workerManagerBean.getWorkerImgs()) {
            workerImg.setWorkerId(workerManagerBean.getUserId());
        }


        workerImgService.saveOrUpdateBatch(workerManagerBean.getWorkerImgs());

        workerPolicyService.lambdaUpdate().eq(WorkerPolicy::getWebsitUserId,workerManagerBean.getId()).remove();


        for (WorkerPolicy workerPolicy : workerManagerBean.getWorkerPolicies()) {
            workerPolicy.setWorkerId(workerManagerBean.getUserId());
            workerPolicy.setWebsitUserId(workerManagerBean.getId());
        }

        workerPolicyService.saveOrUpdateBatch(workerManagerBean.getWorkerPolicies());


        //todo 生成工伤/雇主  自建



    }

    @Transactional(rollbackFor = Exception.class)
    public void examine(WorkerManagerBean workerManagerBean) {

        //todo 检测保险情况 自建，第三方网点

        workerManagerBean.updateById();


        workerManagerBean.getUser().updateById();

        workerImgService.lambdaUpdate().eq(WorkerImg::getWorkerId,workerManagerBean.getUserId())
                .remove();


        for (WorkerImg workerImg : workerManagerBean.getWorkerImgs()) {
            workerImg.setWorkerId(workerManagerBean.getUserId());
        }


        workerImgService.saveOrUpdateBatch(workerManagerBean.getWorkerImgs());

        workerPolicyService.lambdaUpdate().eq(WorkerPolicy::getWebsitUserId,workerManagerBean.getId()).remove();


        for (WorkerPolicy workerPolicy : workerManagerBean.getWorkerPolicies()) {
            workerPolicy.setWorkerId(workerManagerBean.getUserId());
            workerPolicy.setWebsitUserId(workerManagerBean.getId());
        }

        workerPolicyService.saveOrUpdateBatch(workerManagerBean.getWorkerPolicies());


        //todo 生成工伤/雇主  自建
    }
}
